(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react'), require('prop-types')) :
	typeof define === 'function' && define.amd ? define(['exports', 'react', 'prop-types'], factory) :
	(factory((global.GoLib = global.GoLib || {}),global.React,global.PropTypes));
}(this, (function (exports,React,PropTypes) { 'use strict';

React = 'default' in React ? React['default'] : React;
PropTypes = 'default' in PropTypes ? PropTypes['default'] : PropTypes;

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};











var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};











var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

var LINE_WIDTH_FACTOR = 20;

var FONT_SIZE_FACTOR = 2.5;
var STAR_POINT_SIZE_FACTOR = 8;
var BACKGROUND_COLOR = 'rgb(251, 196, 103)';

// BoardDrawer exposes functions for drawing a go board on a 2d context

var BoardDrawer = function () {
  function BoardDrawer(boardSize, width, height) {
    classCallCheck(this, BoardDrawer);

    this.boxSize = width / (boardSize + 1);
    this.lineWidth = this.boxSize / LINE_WIDTH_FACTOR;
    this.fontSize = this.boxSize / FONT_SIZE_FACTOR;
    this.width = width;
    this.height = height;
    this.boardSize = boardSize;
    this.stars = this.calculateStars();
  }

  BoardDrawer.prototype.calculateStars = function calculateStars() {
    var boardSize = this.boardSize,
        boxSize = this.boxSize;

    var starPos = boardSize === 9 ? 3 : 4;
    var single = starPos * boxSize;
    var double = (boardSize + 1) / 2 * boxSize;
    var triple = (boardSize + 1 - starPos) * boxSize;
    var stars = [{ x: double, y: double }];

    if (boardSize % 2 !== 0 && boardSize > 7) {
      stars.push(
      // TopLeft
      { x: single, y: single },
      // Left
      { x: single, y: double },
      // BottomLeft
      { x: single, y: triple },
      // BottomMiddle
      { x: double, y: triple },
      // TopMiddle
      { x: double, y: single },
      // TopRight
      { x: triple, y: single },
      // MiddleRight
      { x: triple, y: double },
      // BottomRight
      { x: triple, y: triple });
    }

    return stars;
  };

  BoardDrawer.prototype.clear = function clear(ctx) {
    var width = this.width,
        height = this.height;


    ctx.save();
    ctx.clearRect(0, 0, width, height);
    ctx.fillStyle = BACKGROUND_COLOR;
    ctx.fillRect(0, 0, width, height);
    ctx.restore();
  };

  BoardDrawer.prototype.stones = function stones(ctx, moves) {
    var lineWidth = this.lineWidth,
        boxSize = this.boxSize;


    ctx.save();
    ctx.lineWidth = lineWidth * 2;

    moves.forEach(function (move, index) {
      var isLastMove = index === moves.length - 1;

      // Draw stone
      ctx.fillStyle = move.color === 'B' ? '#000' : '#FFF';
      ctx.beginPath();
      ctx.arc(move.pos[0] * boxSize, move.pos[1] * boxSize, boxSize / 2, 0, 2 * Math.PI);
      ctx.stroke();
      ctx.fill();

      // Mark last move
      if (isLastMove) {
        ctx.beginPath();
        ctx.fillStyle = move.color === 'B' ? '#FFF' : '#000';
        ctx.lineWidth = lineWidth;
        ctx.arc(move.pos[0] * boxSize, move.pos[1] * boxSize, boxSize / 3.5, 0, 2 * Math.PI);
        ctx.stroke();
      }
    });

    ctx.restore();
  };

  BoardDrawer.prototype.grid = function grid(ctx) {
    var lineWidth = this.lineWidth,
        boxSize = this.boxSize,
        width = this.width,
        height = this.height,
        boardSize = this.boardSize;


    ctx.save();
    ctx.beginPath();
    ctx.lineWidth = lineWidth;

    for (var i = 1; i <= boardSize; i++) {
      ctx.moveTo(i * boxSize, boxSize);
      ctx.lineTo(i * boxSize, height - boxSize);
      ctx.moveTo(boxSize, i * boxSize);
      ctx.lineTo(width - boxSize, i * boxSize);
    }

    ctx.stroke();
    ctx.restore();
  };

  BoardDrawer.prototype.starPoints = function starPoints(ctx) {
    var boxSize = this.boxSize,
        boardSize = this.boardSize,
        stars = this.stars;


    ctx.save();
    ctx.beginPath();
    ctx.fillStyle = '#000';

    stars.forEach(function (pos) {
      ctx.moveTo(pos.x, pos.y);
      ctx.arc(pos.x, pos.y, boxSize / STAR_POINT_SIZE_FACTOR, 0, 2 * Math.PI);
    });

    ctx.fill();
    ctx.restore();
  };

  BoardDrawer.prototype.coordinates = function coordinates(ctx) {
    var boxSize = this.boxSize,
        fontSize = this.fontSize,
        boardSize = this.boardSize;


    ctx.save();
    ctx.font = 'bold ' + fontSize + 'px sans-serif';
    ctx.textAlign = 'center';
    ctx.fillStyle = '#000';

    for (var i = 1; i <= boardSize; i++) {
      ctx.fillText(String.fromCharCode(64 + i), i * boxSize, boxSize / 2);
      ctx.fillText('' + (boardSize - i + 1), boxSize / 2, i * boxSize + 3);
    }

    ctx.restore();
  };

  BoardDrawer.prototype.draw = function draw(ctx, moves) {
    this.clear(ctx);
    this.grid(ctx);
    this.coordinates(ctx);
    this.starPoints(ctx);
    this.stones(ctx, moves);
  };

  return BoardDrawer;
}();

// Board draw's a Go Board
var Board = function (_React$PureComponent) {
  inherits(Board, _React$PureComponent);

  function Board(props) {
    classCallCheck(this, Board);

    var _this = possibleConstructorReturn(this, _React$PureComponent.call(this, props));

    var size = props.size,
        width = props.width,
        height = props.height;


    _this.drawer = new BoardDrawer(size, width, height);
    return _this;
  }

  Board.prototype.componentDidMount = function componentDidMount() {
    var _props = this.props,
        moves = _props.moves,
        width = _props.width,
        height = _props.height;

    var context = this.canvas.getContext('2d');

    this.canvas.width = width;
    this.canvas.height = height;

    if (context) {
      this.context = context;
      this.drawer.draw(this.context, moves);
    }
  };

  Board.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
    var moves = nextProps.moves;


    if (this.context) {
      this.drawer.draw(this.context, moves);
    }
  };

  Board.prototype.render = function render() {
    var _this2 = this;

    return React.createElement('canvas', { ref: function ref(c) {
        return _this2.canvas = c;
      } });
  };

  return Board;
}(React.PureComponent);

Board.propTypes = {
  width: PropTypes.number.isRequired,
  height: PropTypes.number.isRequired,
  size: PropTypes.number,
  moves: PropTypes.arrayOf(PropTypes.shape({
    color: PropTypes.oneOf(['B', 'W'])
  }))
};

Board.defaultProps = {
  size: 19,
  moves: []
};

exports.Board = Board;

Object.defineProperty(exports, '__esModule', { value: true });

})));
